'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.reducer = exports.sendMidiMessage = exports.receiveMidiMessage = exports.setListeningDevices = exports.receiveDeviceList = exports.SEND_MIDI_MESSAGE = exports.RECEIVE_MIDI_MESSAGE = exports.SET_LISTENING_DEVICES = exports.RECEIVE_DEVICE_LIST = undefined;

var _myDuck$createReducer;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = setup;

var _reduxDuck = require('redux-duck');

var _lodash = require('lodash.sortby');

var _lodash2 = _interopRequireDefault(_lodash);

var _deepEqual = require('deep-equal');

var _deepEqual2 = _interopRequireDefault(_deepEqual);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var myDuck = (0, _reduxDuck.createDuck)('midi', 'redux-midi');

var RECEIVE_DEVICE_LIST = exports.RECEIVE_DEVICE_LIST = myDuck.defineType('RECEIVE_DEVICE_LIST');
var SET_LISTENING_DEVICES = exports.SET_LISTENING_DEVICES = myDuck.defineType('SET_LISTENING_DEVICES');
var RECEIVE_MIDI_MESSAGE = exports.RECEIVE_MIDI_MESSAGE = myDuck.defineType('RECEIVE_MIDI_MESSAGE');
var SEND_MIDI_MESSAGE = exports.SEND_MIDI_MESSAGE = myDuck.defineType('SEND_MIDI_MESSAGE');

/**
 * Creates a `RECEIVE_DEVICE_LIST` action.
 * This action is dispatched by the MIDI enhancer at initialization, when devices are connected/disconnected and when devices change state.
 * @param {Array<MIDIDevice>} devices
 */
var receiveDeviceList = exports.receiveDeviceList = myDuck.createAction(RECEIVE_DEVICE_LIST);

/**
 * Creates a `SET_LISTENING_DEVICES` action. Dispatch this action with the `id`s of MIDI input devices you would like to receive messages from.
 * @param {Array<string>} listeningDevices
 */
var setListeningDevices = exports.setListeningDevices = myDuck.createAction(SET_LISTENING_DEVICES);

/**
 * Creates a `RECEIVE_MIDI_MESSAGE` action. This action is dispatched by the MIDI enhancer when a message received from a device you're listening to.
 * @param {MIDIMessage} message
 * @param {Uint8Array} message.data
 * @param {DOMHighResTimeStamp} message.timestamp
 * @param {string} message.device - ID of the source device
 */
var receiveMidiMessage = exports.receiveMidiMessage = myDuck.createAction(RECEIVE_MIDI_MESSAGE);

/**
 * Creates a `SEND_MIDI_MESSAGE` action. Dispatch this action to send a MIDI message to a specified device.
 * @param {MIDIMessage} message
 * @param {Uint8Array} message.data
 * @param {DOMHighResTimeStamp} message.timestamp
 * @param {string} message.device - ID of the destination device
 */
var sendMidiMessage = exports.sendMidiMessage = myDuck.createAction(SEND_MIDI_MESSAGE);

var initialState = {
  devices: [],
  listeningDevices: []
};
/**
 * Reduces MIDI I/O and device discovery actions to state changes.
 * Maintains state in two keys: devices (an array of device objects straight from the underlying Web MIDI API) and `listeningDevices` (an array of device IDs being listened to).
 * @example
 * import { createStore, applyMiddleware, combineReducers } from 'redux';
 * import setup, { reducer } from 'redux-midi';
 * const {inputMiddleware, outputMiddleware} = setup();
 * const store = createStore(combineReducers({midi: reducer}), initialState, applyMiddleware(inputMiddleware, outputMiddleware));
 */
var reducer = exports.reducer = myDuck.createReducer((_myDuck$createReducer = {}, _defineProperty(_myDuck$createReducer, RECEIVE_DEVICE_LIST, function (state, action) {
  return _extends({}, state, {
    devices: action.payload
  });
}), _defineProperty(_myDuck$createReducer, SET_LISTENING_DEVICES, function (state, action) {
  return _extends({}, state, {
    listeningDevices: action.payload
  });
}), _myDuck$createReducer), initialState);

var defaultRequestMIDIAccess = global && global.navigator && typeof global.navigator.requestMIDIAccess === 'function' && global.navigator.requestMIDIAccess.bind(global.navigator) || function () {
  return Promise.reject(new Error('Web MIDI API not available'));
};

/**
 * Create a pair of Redux {@link https://github.com/reactjs/redux/blob/master/docs/Glossary.md#middleware|middleware} functions wrapping MIDI I/O and device discovery.
 * The input middleware dispatches RECEIVE_DEVICE_LIST whenever the list of MIDI devices changes and RECEIVE_MIDI_MESSAGE when MIDI messages are received on devices that are being listened to.
 * The output middleware sends MIDI messages to output devices as a side effect of SEND_MIDI_MESSAGE actions.
 * @param {MIDIOptions} [$0.midiOptions] - Options with which to invoke `requestMIDIAccess`.
 * @param {string} [$0.stateKey='midi'] - The state key at which redux-midi's reducer is mounted.
 * @param {function(MIDIOptions): Promise<MIDIAccess>} [$0.requestMIDIAccess=navigator.requestMIDIAccess] - Web MIDI API entry point.
 * @example
 * import { createStore, applyMiddleware, combineReducers } from 'redux';
 * import setup, { reducer } from 'redux-midi';
 * const {inputMiddleware, outputMiddleware} = setup();
 * const store = createStore(combineReducers({midi: reducer}), initialState, applyMiddleware(inputMiddleware, outputMiddleware));
 */
function setup(_ref) {
  var midiOptions = _ref.midiOptions,
      _ref$stateKey = _ref.stateKey,
      stateKey = _ref$stateKey === undefined ? 'midi' : _ref$stateKey,
      _ref$requestMIDIAcces = _ref.requestMIDIAccess,
      requestMIDIAccess = _ref$requestMIDIAcces === undefined ? defaultRequestMIDIAccess : _ref$requestMIDIAcces;

  var midiAccess = void 0;
  var requestMIDIAccessOnce = function requestMIDIAccessOnce() {
    if (midiAccess) return Promise.resolve(midiAccess);

    midiAccess = requestMIDIAccess(midiOptions).then(function (receivedMidiAccess) {
      midiAccess = receivedMidiAccess;
      return midiAccess;
    });
    return midiAccess;
  };

  var inputMiddleware = function inputMiddleware(_ref2) {
    var dispatch = _ref2.dispatch,
        getState = _ref2.getState;

    return function (next) {
      requestMIDIAccessOnce().then(function () {
        var sendDeviceList = function sendDeviceList() {
          var devices = (0, _lodash2.default)([].concat(_toConsumableArray(midiAccess.inputs.values()), _toConsumableArray(midiAccess.outputs.values())).map(function (device) {
            return {
              id: device.id,
              manufacturer: device.manufacturer,
              name: device.name,
              type: device.type,
              version: device.version,
              state: device.state,
              connection: device.connection
            };
          }), 'id');
          dispatch(receiveDeviceList(devices));
        };

        midiAccess.onstatechange = function () {
          return sendDeviceList();
        };
        Promise.resolve().then(sendDeviceList);
      });

      return function (action) {
        var toListen = [];
        var toUnlisten = [];
        var prevState = getState()[stateKey] || initialState;
        action = next(action);
        var state = getState()[stateKey];

        if (state.listeningDevices !== prevState.listeningDevices) {
          var prev = new Set(prevState ? prevState.listeningDevices : []);
          var _next = new Set(state.listeningDevices);
          toUnlisten.push.apply(toUnlisten, _toConsumableArray(prevState.listeningDevices.filter(function (dev) {
            return !_next.has(dev);
          })));
          toListen.push.apply(toListen, _toConsumableArray(state.listeningDevices.filter(function (dev) {
            return !prev.has(dev);
          })));
        }

        if (state.devices !== prevState.devices) {
          var _prev = new Set(prevState.devices.map(function (device) {
            return device.id;
          }));
          var _next2 = new Set(state.devices.map(function (device) {
            return device.id;
          }));
          toListen.push.apply(toListen, _toConsumableArray(state.listeningDevices.filter(function (device) {
            return midiAccess.inputs.has(device) && _next2.has(device) && !_prev.has(device);
          })));
        }

        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = toUnlisten[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var device = _step.value;

            if (midiAccess.inputs.has(device)) {
              midiAccess.inputs.get(device).onmidimessage = null;
            }
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }

        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
          var _loop = function _loop() {
            var device = _step2.value;

            if (midiAccess.inputs.has(device)) {
              midiAccess.inputs.get(device).onmidimessage = function (_ref3) {
                var receivedTime = _ref3.receivedTime,
                    timeStamp = _ref3.timeStamp,
                    timestamp = _ref3.timestamp,
                    data = _ref3.data;

                timestamp = [receivedTime, timeStamp, timestamp].filter(function (x) {
                  return x !== undefined;
                })[0];
                dispatch(receiveMidiMessage({ timestamp: timestamp, data: data, device: device }));
              };
            }
          };

          for (var _iterator2 = toListen[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            _loop();
          }
        } catch (err) {
          _didIteratorError2 = true;
          _iteratorError2 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
              _iterator2.return();
            }
          } finally {
            if (_didIteratorError2) {
              throw _iteratorError2;
            }
          }
        }

        return action;
      };
    };
  };

  var outputMiddleware = function outputMiddleware() {
    return function (next) {
      return function (action) {
        action = next(action);
        if (action.type === SEND_MIDI_MESSAGE) {
          var _action = action,
              payload = _action.payload;
          var timestamp = payload.timestamp,
              data = payload.data,
              device = payload.device;

          var send = function send() {
            var _midiAccess = midiAccess,
                outputs = _midiAccess.outputs;

            if (outputs.has(device)) {
              outputs.get(device).send(data, timestamp);
            }
          };
          if (midiAccess && !midiAccess.then) send();else requestMIDIAccessOnce().then(send);
        }
        return action;
      };
    };
  };

  return { inputMiddleware: inputMiddleware, outputMiddleware: outputMiddleware };
}